﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Concats two vectors on the type stack.
  /// </summary>
  public abstract class VectorConcatExpression<T> : StatelessExpression {
    protected bool Eval(IInternalPushInterpreter interpter, IPushStack<List<T>> stack) {
      if (stack.Count < 2 ||
          stack.Top.Count + stack[1].Count > interpter.Configuration.MaxVectorLength)
        return false;

      var first = stack.Pop();
      var second = stack.Top;
      var result = new T[first.Count + second.Count];

      first.CopyTo(result, 0);
      second.CopyTo(result, first.Count);
      stack.SetTop(new List<T>(result));

      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].CONCAT")]
  public class IntegerVectorConcatExpression : VectorConcatExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].CONCAT")]
  public class FloatVectorConcatExpression : VectorConcatExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].CONCAT")]
  public class BooleanVectorConcatExpression : VectorConcatExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].CONCAT")]
  public class StringVectorConcatExpression : VectorConcatExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack);
    }
  }
}
