﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Replaces, in the top type vector, item at index(from integer stack) with the first lit-type item.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorSetExpression<T> : StatelessExpression {
    protected bool Eval(
      IInternalPushInterpreter interpreter,
      IPushStack<List<T>> vectorStack,
      IPushStack<T> literalStack,
      bool isLiteralTypeInteger = false) {
      if (vectorStack.IsEmpty ||
          vectorStack.Top.Count == 0 ||
          literalStack.IsEmpty ||
          (isLiteralTypeInteger && interpreter.IntegerStack.Count < 2) ||
          (!isLiteralTypeInteger && interpreter.IntegerStack.IsEmpty))
        return false;

      T literal;
      int index;

      if (isLiteralTypeInteger) {
        literal = literalStack.Top;
        index = (int)interpreter.IntegerStack[1];
        interpreter.IntegerStack.Remove(2);
      } else {
        literal = literalStack.Pop();
        index = (int)interpreter.IntegerStack.Pop();
      }

      var vector = vectorStack.Pop();
      index = index % vector.Count;

      if (index < 0)
        index *= -1;

      vector[index] = literal;
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].SET", StackTypes.Integer)]
  public class IntegerVectorSetExpression : VectorSetExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack, interpreter.IntegerStack, true);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].SET", StackTypes.Float | StackTypes.Integer)]
  public class FloatVectorSetExpression : VectorSetExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].SET", StackTypes.Boolean | StackTypes.Integer)]
  public class BooleanVectorSetExpression : VectorSetExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].SET", StackTypes.String | StackTypes.Integer)]
  public class StringVectorSetExpression : VectorSetExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}
