﻿using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Takes the first N items from the type stack, where N is from the integer stack
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorTakeExpression<T> : StatelessExpression {
    protected bool Eval(IInternalPushInterpreter interpreter, IPushStack<List<T>> vectorStack) {
      if (vectorStack.IsEmpty || interpreter.IntegerStack.IsEmpty)
        return false;

      var count = (int)interpreter.IntegerStack.Pop();

      if (vectorStack.Top.Count == 0)
        return true;

      count %= vectorStack.Top.Count;

      if (count < 0)
        count *= -1;

      var result = vectorStack.Top.GetRange(0, count);
      vectorStack.SetTop(result);

      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].TAKE", StackTypes.Integer)]
  public class IntegerVectorTakeExpression : VectorTakeExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].TAKE", StackTypes.Integer)]
  public class FloatVectorTakeExpression : VectorTakeExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].TAKE", StackTypes.Integer)]
  public class BooleanVectorTakeExpression : VectorTakeExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].TAKE", StackTypes.Integer)]
  public class StringVectorTakeExpression : VectorTakeExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack);
    }
  }
}
